@@ -25,6 +25,10 @@
 
 #include "udfread.h"
 
+#ifdef HAVE_UDFREAD_VERSION_H
+#include "udfread-version.h"
+#endif
+
 #include "blockinput.h"
 #include "default_blockinput.h"
 #include "ecma167.h"
@@ -139,7 +143,7 @@ static void *_safe_realloc(void *p, size_t s)
 #define utf16lo_to_utf8(out, out_pos, out_size, ch) \
   do {                                              \
     if (ch < 0x80) {                                \
-      out[out_pos++] = ch;                          \
+      out[out_pos++] = (uint8_t)ch;                 \
     } else {                                        \
       out_size++;                                   \
       out = (uint8_t *)_safe_realloc(out, out_size);\
@@ -508,7 +512,11 @@ static int _map_metadata_partition(udfread_block_input *input,
             continue;
         }
 
-        if (fe->file_type == UDF_FT_METADATA) {
+        if (fe->content_inline) {
+            udf_error("invalid metadata file (content inline)\n");
+        } else if (!fe->num_ad) {
+            udf_error("invalid metadata file (no allocation descriptors)\n");
+        } else if (fe->file_type == UDF_FT_METADATA) {
             part->p[1].lba = pd->start_block + fe->data.ad[0].lba;
             udf_log("metadata file at lba %u\n", part->p[1].lba);
         } else if (fe->file_type == UDF_FT_METADATA_MIRROR) {
@@ -693,6 +701,10 @@ udfread *udfread_init(void)
         enable_log = 1;
     }
 
+#ifdef HAVE_UDFREAD_VERSION_H
+    udf_log("libudfread " UDFREAD_VERSION_STRING "\n");
+#endif
+
     return (udfread *)calloc(1, sizeof(udfread));
 }
 
@@ -826,8 +838,9 @@ static struct file_entry *_read_file_entry(udfread *udf,
     free(buf);
 
     /* read possible additional allocation extents */
-    if (fe && fe->num_ad > 0) {
-        while (fe->data.ad[fe->num_ad - 1].extent_type == ECMA_AD_EXTENT_AD) {
+    if (fe) {
+        while (fe->num_ad > 0 &&
+               fe->data.ad[fe->num_ad - 1].extent_type == ECMA_AD_EXTENT_AD) {
 
             icb = &fe->data.ad[fe->num_ad - 1];
             udf_log("_read_file_entry: reading allocation extent @%u\n", icb->lba);
@@ -840,7 +853,7 @@ static struct file_entry *_read_file_entry(udfread *udf,
 
             if (tag_id != ECMA_AllocationExtentDescriptor) {
                 free(buf);
-                udf_error("_read_file_entry: unexpected tag %u (expected ECMA_AllocationExtentDescriptor)\n", tag_id);
+                udf_error("_read_file_entry: unexpected tag %d (expected ECMA_AllocationExtentDescriptor)\n", tag_id);
                 break;
             }
 
@@ -1391,7 +1404,7 @@ static uint32_t _file_lba(UDFFILE *p, uint32_t file_block)
 
             if (ad[i].extent_type != ECMA_AD_EXTENT_NORMAL) {
                 if (ad[i].extent_type == ECMA_AD_EXTENT_AD) {
-                    udf_error("unsupported allocation desriptor: extent type %u\n", ad[i].extent_type);
+                    udf_error("unsupported allocation descriptor: extent type %u\n", ad[i].extent_type);
                 }
                 return 0;
             }
@@ -1479,12 +1492,16 @@ uint32_t udfread_read_blocks(UDFFILE *p, void *buf, uint32_t file_block, uint32_
 
 static ssize_t _read(UDFFILE *p, void *buf, size_t bytes)
 {
-    /* start from middle of block ? */
+    /* start from middle of block ?
+     * maximal file size, i.e. position, is 2^32 * block size
+     */
+
     size_t pos_off = p->pos % UDF_BLOCK_SIZE;
+    uint32_t file_block = (uint32_t)(p->pos / UDF_BLOCK_SIZE);
     if (pos_off) {
         size_t chunk_size = UDF_BLOCK_SIZE - pos_off;
         if (!p->block_valid) {
-            if (udfread_read_blocks(p, p->block, p->pos / UDF_BLOCK_SIZE, 1, 0) != 1) {
+            if (udfread_read_blocks(p, p->block, file_block, 1, 0) != 1) {
                 return -1;
             }
             p->block_valid = 1;
@@ -1500,7 +1517,7 @@ static ssize_t _read(UDFFILE *p, void *buf, size_t bytes)
     /* read full block(s) ? */
     if (bytes >= UDF_BLOCK_SIZE) {
         uint32_t num_blocks = bytes / UDF_BLOCK_SIZE;
-        num_blocks = udfread_read_blocks(p, buf, p->pos / UDF_BLOCK_SIZE, num_blocks, 0);
+        num_blocks = udfread_read_blocks(p, buf, file_block, num_blocks, 0);
         if (num_blocks < 1) {
             return -1;
         }
@@ -1509,7 +1526,7 @@ static ssize_t _read(UDFFILE *p, void *buf, size_t bytes)
     }
 
     /* read beginning of a block */
-    if (udfread_read_blocks(p, p->block, p->pos / UDF_BLOCK_SIZE, 1, 0) != 1) {
+    if (udfread_read_blocks(p, p->block, file_block, 1, 0) != 1) {
         return -1;
     }
     p->block_valid = 1;
