@@ -21,13 +21,12 @@ package javax.tv.xlet;
 
 public class XletStateChangeException extends Exception
 {
+    public XletStateChangeException() {
+    }
+
+    public XletStateChangeException(String s) {
+        super(s);
+    }
+
     private static final long serialVersionUID = 6559278303854606991L;
-
-    public XletStateChangeException() 
-    { 
-    }
-
-    public XletStateChangeException(String s) 
-    { 
-    }
 }
