@@ -56,6 +56,8 @@ import org.dvb.media.SubtitleListener;
 import org.dvb.media.SubtitleNotAvailableEvent;
 import org.dvb.media.SubtitleNotSelectedEvent;
 import org.dvb.media.SubtitleSelectedEvent;
+import org.dvb.media.VideoFormatListener;
+import org.dvb.media.VideoFormatEvent;
 
 public class BDJListeners {
     private LinkedList listeners = new LinkedList();
@@ -139,7 +141,7 @@ public class BDJListeners {
         }
     }
 
-    private class PSR102Status {
+    private static class PSR102Status {
         private PSR102Status(int value) {
             this.value = value;
         }
@@ -220,6 +222,9 @@ public class BDJListeners {
                        event instanceof SubtitleNotSelectedEvent || event instanceof SubtitleSelectedEvent) {
                 ((SubtitleListener)listener).subtitleStatusChanged((EventObject)event);
 
+            } else if (event instanceof VideoFormatEvent) {
+                ((VideoFormatListener)listener).receiveVideoFormatEvent((VideoFormatEvent)event);
+
             } else if (event instanceof PSR102Status) {
                 ((StatusListener)listener).receive(((PSR102Status)event).value);
 
