@@ -140,7 +140,8 @@ public class AppEntry implements AppAttributes {
     }
 
     public boolean isVisible() {
-        return ((visibility & VISIBLE_TO_USERS) != 0);
+        // Annex S
+        return false;
     }
 
     public boolean isDiscBound() {
@@ -159,7 +160,7 @@ public class AppEntry implements AppAttributes {
         this.profiles = profiles;
         this.priority = priority;
         this.binding = binding;
-        this.visibility = visibility;
+        //this.visibility = visibility;
         this.names = names;
         if ((iconLocator != null) && (iconLocator.length() > 0)) {
             try {
@@ -216,7 +217,7 @@ public class AppEntry implements AppAttributes {
     private AppProfile[] profiles;
     private short priority;
     private int binding;
-    private int visibility;
+    //private int visibility;
     private String[][] names;
     private AppIcon icon;
     private String basePath;
