@@ -154,6 +154,12 @@ static int _pl_cmp(MPLS_PL *pl1, MPLS_PL *pl2)
     if (pl1->mark_count != pl2->mark_count) {
         return 1;
     }
+    if (pl1->sub_count != pl2->sub_count) {
+        return 1;
+    }
+    if (pl1->ext_sub_count != pl2->ext_sub_count) {
+        return 1;
+    }
 
     for (ii = 0; ii < pl1->mark_count; ii++) {
         if (_pm_cmp(&pl1->play_mark[ii], &pl2->play_mark[ii])) {
@@ -284,7 +290,9 @@ static int _cmp_audio_props(const MPLS_PL *p1, const MPLS_PL *p2)
     return hda2 - hda1;
 }
 
-static int _pl_guess_main_title(MPLS_PL *p1, MPLS_PL *p2)
+static int _pl_guess_main_title(MPLS_PL *p1, MPLS_PL *p2,
+                                const char *mpls_id1, const char *mpls_id2,
+                                const char *known_mpls_ids)
 {
     uint32_t d1 = _pl_duration(p1);
     uint32_t d2 = _pl_duration(p2);
@@ -298,23 +306,38 @@ static int _pl_guess_main_title(MPLS_PL *p1, MPLS_PL *p2)
         int chap_diff = chap2 - chap1;
         if ((chap1 < 2 || chap2 < 2) && (chap_diff < -5 || chap_diff > 5)) {
             /* chapter count differs by more than 5 */
-            BD_DEBUG(DBG_MAIN_PL, "main title: chapter count difference %d\n", chap_diff);
+            BD_DEBUG(DBG_MAIN_PL, "main title (%s,%s): chapter count difference %d\n",
+                     mpls_id1, mpls_id2, chap_diff);
             return chap_diff;
         }
 
         /* Check video: prefer HD over SD, H.264/VC1 over MPEG1/2 */
         int vid_diff = _cmp_video_props(p1, p2);
         if (vid_diff) {
-            BD_DEBUG(DBG_MAIN_PL, "main title: video properties difference %d\n", vid_diff);
+            BD_DEBUG(DBG_MAIN_PL, "main title (%s,%s): video properties difference %d\n",
+                     mpls_id1, mpls_id2, vid_diff);
             return vid_diff;
         }
 
         /* compare audio: prefer HD audio */
         int aud_diff = _cmp_audio_props(p1, p2);
         if (aud_diff) {
-            BD_DEBUG(DBG_MAIN_PL, "main title: audio properties difference %d\n", aud_diff);
+            BD_DEBUG(DBG_MAIN_PL, "main title (%s,%s): audio properties difference %d\n",
+                     mpls_id1, mpls_id2, aud_diff);
             return aud_diff;
         }
+
+        /* prefer "known good" playlists */
+        if (known_mpls_ids) {
+            int known1 = !!str_strcasestr(known_mpls_ids, mpls_id1);
+            int known2 = !!str_strcasestr(known_mpls_ids, mpls_id2);
+            int known_diff = known2 - known1;
+            if (known_diff) {
+                BD_DEBUG(DBG_MAIN_PL, "main title (%s,%s): prefer \"known\" playlist %s\n",
+                         mpls_id1, mpls_id2, known_diff < 0 ? mpls_id1 : mpls_id2);
+                return known_diff;
+            }
+        }
     }
 
     /* compare playlist duration, select longer playlist */
@@ -340,8 +363,9 @@ NAV_TITLE_LIST* nav_get_title_list(BD_DISC *disc, uint32_t flags, uint32_t min_t
     MPLS_PL *pl = NULL;
     unsigned int ii, pl_list_size = 0;
     int res;
-    NAV_TITLE_LIST *title_list;
+    NAV_TITLE_LIST *title_list = NULL;
     unsigned int title_info_alloc = 100;
+    char *known_mpls_ids;
 
     dir = disc_open_dir(disc, "BDMV" DIR_SEP "PLAYLIST");
     if (dir == NULL) {
@@ -349,7 +373,21 @@ NAV_TITLE_LIST* nav_get_title_list(BD_DISC *disc, uint32_t flags, uint32_t min_t
     }
 
     title_list = calloc(1, sizeof(NAV_TITLE_LIST));
+    if (!title_list) {
+        dir_close(dir);
+        return NULL;
+    }
     title_list->title_info = calloc(title_info_alloc, sizeof(NAV_TITLE_INFO));
+    if (!title_list->title_info) {
+        X_FREE(title_list);
+        dir_close(dir);
+        return NULL;
+    }
+
+    known_mpls_ids = disc_property_get(disc, DISC_PROPERTY_MAIN_FEATURE);
+    if (!known_mpls_ids) {
+        known_mpls_ids = disc_property_get(disc, DISC_PROPERTY_PLAYLISTS);
+    }
 
     ii = 0;
     for (res = dir_read(dir, &ent); !res; res = dir_read(dir, &ent)) {
@@ -371,16 +409,16 @@ NAV_TITLE_LIST* nav_get_title_list(BD_DISC *disc, uint32_t flags, uint32_t min_t
         if (pl != NULL) {
             if ((flags & TITLES_FILTER_DUP_TITLE) &&
                 !_filter_dup(pl_list, ii, pl)) {
-                mpls_free(pl);
+                mpls_free(&pl);
                 continue;
             }
             if ((flags & TITLES_FILTER_DUP_CLIP) && !_filter_repeats(pl, 2)) {
-                mpls_free(pl);
+                mpls_free(&pl);
                 continue;
             }
             if (min_title_length > 0 &&
                 _pl_duration(pl) < min_title_length*45000) {
-                mpls_free(pl);
+                mpls_free(&pl);
                 continue;
             }
             if (ii >= title_info_alloc) {
@@ -400,7 +438,10 @@ NAV_TITLE_LIST* nav_get_title_list(BD_DISC *disc, uint32_t flags, uint32_t min_t
             if (_filter_dup(pl_list, ii, pl) &&
                 _filter_repeats(pl, 2)) {
 
-                if (_pl_guess_main_title(pl_list[ii], pl_list[title_list->main_title_idx]) <= 0) {
+                if (_pl_guess_main_title(pl_list[ii], pl_list[title_list->main_title_idx],
+                                         ent.d_name,
+                                         title_list->title_info[title_list->main_title_idx].name,
+                                         known_mpls_ids) <= 0) {
                     title_list->main_title_idx = ii;
                 }
             }
@@ -417,16 +458,19 @@ NAV_TITLE_LIST* nav_get_title_list(BD_DISC *disc, uint32_t flags, uint32_t min_t
 
     title_list->count = ii;
     for (ii = 0; ii < title_list->count; ii++) {
-        mpls_free(pl_list[ii]);
+        mpls_free(&pl_list[ii]);
     }
+    X_FREE(known_mpls_ids);
     X_FREE(pl_list);
     return title_list;
 }
 
-void nav_free_title_list(NAV_TITLE_LIST *title_list)
+void nav_free_title_list(NAV_TITLE_LIST **title_list)
 {
-    X_FREE(title_list->title_info);
-    X_FREE(title_list);
+    if (*title_list) {
+        X_FREE((*title_list)->title_info);
+        X_FREE((*title_list));
+    }
 }
 
 /*
@@ -465,7 +509,6 @@ _fill_mark(NAV_TITLE *title, NAV_MARK *mark, int entry)
 
     plm = &pl->play_mark[entry];
 
-    mark->plm = plm;
     mark->mark_type = plm->mark_type;
     mark->clip_ref = plm->play_item_ref;
     clip = &title->clip_list.clip[mark->clip_ref];
@@ -566,8 +609,7 @@ static void _fill_clip(NAV_TITLE *title,
     strncpy(&clip->name[5], ".m2ts", 6);
     clip->clip_id = atoi(mpls_clip[clip->angle].clip_id);
 
-    clpi_free(clip->cl);
-    clip->cl = NULL;
+    clpi_free(&clip->cl);
 
     file = str_printf("%s.clpi", mpls_clip[clip->angle].clip_id);
     if (file) {
@@ -685,18 +727,16 @@ NAV_TITLE* nav_title_open(BD_DISC *disc, const char *playlist, unsigned angle)
     return title;
 }
 
-void nav_title_close(NAV_TITLE *title)
+static
+void _nav_title_close(NAV_TITLE *title)
 {
     unsigned ii, ss;
 
-    if (!title)
-        return;
-
     if (title->sub_path) {
         for (ss = 0; ss < title->sub_path_count; ss++) {
             if (title->sub_path[ss].clip_list.clip) {
                 for (ii = 0; ii < title->sub_path[ss].clip_list.count; ii++) {
-                    clpi_free(title->sub_path[ss].clip_list.clip[ii].cl);
+                    clpi_free(&title->sub_path[ss].clip_list.clip[ii].cl);
                 }
                 X_FREE(title->sub_path[ss].clip_list.clip);
             }
@@ -706,17 +746,25 @@ void nav_title_close(NAV_TITLE *title)
 
     if (title->clip_list.clip) {
         for (ii = 0; ii < title->clip_list.count; ii++) {
-            clpi_free(title->clip_list.clip[ii].cl);
+            clpi_free(&title->clip_list.clip[ii].cl);
         }
         X_FREE(title->clip_list.clip);
     }
 
-    mpls_free(title->pl);
+    mpls_free(&title->pl);
     X_FREE(title->chap_list.mark);
     X_FREE(title->mark_list.mark);
     X_FREE(title);
 }
 
+void nav_title_close(NAV_TITLE **title)
+{
+    if (*title) {
+        _nav_title_close(*title);
+        *title = NULL;
+    }
+}
+
 // Search for random access point closest to the requested packet
 // Packets are 192 byte TS packets
 NAV_CLIP* nav_chapter_search(NAV_TITLE *title, unsigned chapter, uint32_t *clip_pkt, uint32_t *out_pkt)
@@ -776,6 +824,24 @@ NAV_CLIP* nav_mark_search(NAV_TITLE *title, unsigned mark, uint32_t *clip_pkt, u
     return clip;
 }
 
+void nav_clip_packet_search(NAV_CLIP *clip, uint32_t pkt, uint32_t *clip_pkt, uint32_t *clip_time)
+{
+    *clip_time = clip->in_time;
+    if (clip->cl != NULL) {
+        *clip_pkt = clpi_access_point(clip->cl, pkt, 0, 0, clip_time);
+        if (*clip_pkt < clip->start_pkt) {
+            *clip_pkt = clip->start_pkt;
+        }
+        if (*clip_time && *clip_time < clip->in_time) {
+            /* EP map does not store lowest 8 bits of timestamp */
+            *clip_time = clip->in_time;
+        }
+
+    } else {
+        *clip_pkt = clip->start_pkt;
+    }
+}
+
 // Search for random access point closest to the requested packet
 // Packets are 192 byte TS packets
 // pkt is relative to the beginning of the title
@@ -801,14 +867,7 @@ NAV_CLIP* nav_packet_search(NAV_TITLE *title, uint32_t pkt, uint32_t *clip_pkt,
         *clip_pkt = clip->end_pkt;
     } else {
         clip = &title->clip_list.clip[ii];
-        if (clip->cl != NULL) {
-            *clip_pkt = clpi_access_point(clip->cl, pkt - pos + clip->start_pkt, 0, 0, out_time);
-            if (*clip_pkt < clip->start_pkt) {
-                *clip_pkt = clip->start_pkt;
-            }
-        } else {
-            *clip_pkt = clip->start_pkt;
-        }
+        nav_clip_packet_search(clip, pkt - pos + clip->start_pkt, clip_pkt, out_time);
     }
     if(*out_time < clip->in_time)
         *out_time = 0;
@@ -878,22 +937,14 @@ NAV_CLIP* nav_time_search(NAV_TITLE *title, uint32_t tick, uint32_t *clip_pkt, u
         *clip_pkt = clip->end_pkt;
     } else {
         clip = &title->clip_list.clip[ii];
-        if (clip->cl != NULL) {
-            *clip_pkt = clpi_lookup_spn(clip->cl, tick - pos + pi->in_time, 1,
-                      title->pl->play_item[clip->ref].clip[clip->angle].stc_id);
-            if (*clip_pkt < clip->start_pkt) {
-                *clip_pkt = clip->start_pkt;
-            }
-        } else {
-            *clip_pkt = clip->start_pkt;
-        }
+        nav_clip_time_search(clip, tick - pos + pi->in_time, clip_pkt, out_pkt);
     }
     *out_pkt = clip->title_pkt + *clip_pkt - clip->start_pkt;
     return clip;
 }
 
 // Search for random access point closest to the requested time
-// Time is in 45khz ticks relative to the beginning of a specific clip
+// Time is in 45khz ticks, between clip in_time and out_time.
 void nav_clip_time_search(NAV_CLIP *clip, uint32_t tick, uint32_t *clip_pkt, uint32_t *out_pkt)
 {
     if (tick >= clip->out_time) {
@@ -902,6 +953,10 @@ void nav_clip_time_search(NAV_CLIP *clip, uint32_t tick, uint32_t *clip_pkt, uin
         if (clip->cl != NULL) {
             *clip_pkt = clpi_lookup_spn(clip->cl, tick, 1,
                clip->title->pl->play_item[clip->ref].clip[clip->angle].stc_id);
+            if (*clip_pkt < clip->start_pkt) {
+                *clip_pkt = clip->start_pkt;
+            }
+
         } else {
             *clip_pkt = clip->start_pkt;
         }
