@@ -22,7 +22,7 @@
 
 #include "util/attributes.h"
 
-#include "mpls_parse.h"
+#include "mpls_data.h"
 #include "clpi_data.h"
 
 struct bd_disc;
@@ -50,11 +50,8 @@ struct nav_mark_s
     uint32_t title_pkt;
     uint32_t title_time;
     uint32_t duration;
-
-    MPLS_PLM *plm;
 };
 
-
 typedef struct nav_mark_list_s NAV_MARK_LIST;
 struct nav_mark_list_s
 {
@@ -141,7 +138,7 @@ struct nav_title_list_s
 
 BD_PRIVATE uint8_t nav_lookup_aspect(NAV_CLIP *clip, int pid);
 BD_PRIVATE NAV_TITLE* nav_title_open(struct bd_disc *disc, const char *playlist, unsigned angle) BD_ATTR_MALLOC;
-BD_PRIVATE void nav_title_close(NAV_TITLE *title);
+BD_PRIVATE void nav_title_close(NAV_TITLE **title);
 BD_PRIVATE NAV_CLIP* nav_next_clip(NAV_TITLE *title, NAV_CLIP *clip);
 BD_PRIVATE NAV_CLIP* nav_packet_search(NAV_TITLE *title, uint32_t pkt, uint32_t *clip_pkt,
                                        uint32_t *out_pkt, uint32_t *out_time);
@@ -155,6 +152,6 @@ BD_PRIVATE uint32_t nav_angle_change_search(NAV_CLIP *clip, uint32_t pkt, uint32
 BD_PRIVATE NAV_CLIP* nav_set_angle(NAV_TITLE *title, NAV_CLIP *clip, unsigned angle);
 
 BD_PRIVATE NAV_TITLE_LIST* nav_get_title_list(struct bd_disc *disc, uint32_t flags, uint32_t min_title_length) BD_ATTR_MALLOC;
-BD_PRIVATE void nav_free_title_list(NAV_TITLE_LIST *title_list);
+BD_PRIVATE void nav_free_title_list(NAV_TITLE_LIST **title_list);
 
 #endif // _NAVIGATION_H_
