@@ -104,7 +104,7 @@ BD_PRIVATE void bd_registers_free(BD_REGISTERS *);
  * @param reg  register number
  * @return value stored in register, -1 on error (invalid register number)
  */
-uint32_t bd_gpr_read(BD_REGISTERS *, int reg);
+uint32_t bd_gpr_read(BD_REGISTERS *, unsigned int reg);
 
 /**
  *
@@ -115,7 +115,7 @@ uint32_t bd_gpr_read(BD_REGISTERS *, int reg);
  * @param val  new value for register
  * @return 0 on success, -1 on error (invalid register number)
  */
-int bd_gpr_write(BD_REGISTERS *, int reg, uint32_t val);
+int bd_gpr_write(BD_REGISTERS *, unsigned int reg, uint32_t val);
 
 
 /*
@@ -130,7 +130,7 @@ int bd_gpr_write(BD_REGISTERS *, int reg, uint32_t val);
  * @param reg  register number
  * @return value stored in register, -1 on error (invalid register number)
  */
-uint32_t bd_psr_read(BD_REGISTERS *, int reg);
+uint32_t bd_psr_read(BD_REGISTERS *, unsigned int reg);
 
 /**
  *
@@ -143,7 +143,7 @@ uint32_t bd_psr_read(BD_REGISTERS *, int reg);
  * @param val  new value for register
  * @return 0 on success, -1 on error (invalid register number)
  */
-int bd_psr_write(BD_REGISTERS *, int reg, uint32_t val);
+int bd_psr_write(BD_REGISTERS *, unsigned int reg, uint32_t val);
 
 /**
  *
@@ -159,7 +159,7 @@ int bd_psr_write(BD_REGISTERS *, int reg, uint32_t val);
  * @param mask  bit mask. bits to be written are set to 1.
  * @return 0 on success, -1 on error (invalid register number)
  */
-int bd_psr_write_bits(BD_REGISTERS *, int reg, uint32_t val, uint32_t mask);
+BD_PRIVATE int bd_psr_write_bits(BD_REGISTERS *, unsigned int reg, uint32_t val, uint32_t mask);
 
 /**
  *
@@ -172,7 +172,7 @@ int bd_psr_write_bits(BD_REGISTERS *, int reg, uint32_t val, uint32_t mask);
  * @param val  new value for register
  * @return 0 on success, -1 on error (invalid register number)
  */
-int bd_psr_setting_write(BD_REGISTERS *, int reg, uint32_t val);
+BD_PRIVATE int bd_psr_setting_write(BD_REGISTERS *, unsigned int reg, uint32_t val);
 
 /**
  *
@@ -180,7 +180,7 @@ int bd_psr_setting_write(BD_REGISTERS *, int reg, uint32_t val);
  *
  * @param registers  BD_REGISTERS object
  */
-void bd_psr_lock(BD_REGISTERS *);
+BD_PRIVATE void bd_psr_lock(BD_REGISTERS *);
 
 /**
  *
@@ -188,7 +188,7 @@ void bd_psr_lock(BD_REGISTERS *);
  *
  * @param registers  BD_REGISTERS object
  */
-void bd_psr_unlock(BD_REGISTERS *);
+BD_PRIVATE void bd_psr_unlock(BD_REGISTERS *);
 
 /**
  *
@@ -198,7 +198,7 @@ void bd_psr_unlock(BD_REGISTERS *);
  *
  * @param registers  BD_REGISTERS object
  */
-void bd_psr_save_state(BD_REGISTERS *);
+BD_PRIVATE void bd_psr_save_state(BD_REGISTERS *);
 
 /**
  *
@@ -209,7 +209,7 @@ void bd_psr_save_state(BD_REGISTERS *);
  *
  * @param registers  BD_REGISTERS object
  */
-void bd_psr_restore_state(BD_REGISTERS *);
+BD_PRIVATE void bd_psr_restore_state(BD_REGISTERS *);
 
 /**
  *
@@ -219,7 +219,7 @@ void bd_psr_restore_state(BD_REGISTERS *);
  *
  * @param registers  BD_REGISTERS object
  */
-void bd_psr_reset_backup_registers(BD_REGISTERS *);
+BD_PRIVATE void bd_psr_reset_backup_registers(BD_REGISTERS *);
 
 
 /*
@@ -234,9 +234,9 @@ void bd_psr_reset_backup_registers(BD_REGISTERS *);
 
 /* event data */
 typedef struct {
-    int      ev_type; /* event type */
+    unsigned ev_type; /* event type */
 
-    int      psr_idx; /* register index */
+    unsigned psr_idx; /* register index */
     uint32_t old_val; /* old value of register */
     uint32_t new_val; /* new value of register */
 } BD_PSR_EVENT;
